/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */


#include "ChangeFramerate.h"
#include "utility.h"

/*
 *
 */
ChangeFramerate::ChangeFramerate()
:Action()
{
}

/*
 *
 */
ChangeFramerate::~ChangeFramerate()
{
}

/*
 *
 */
bool ChangeFramerate::execute(Document &doc, double src_fps, double dest_fps)
{
	Gtk::TreeNodeChildren rows = doc.get_subtitle_model()->children();

	for(Gtk::TreeIter it = rows.begin(); it; ++it)
	{
		SubtitleModifier subtitle(it);

		SubtitleTime start = change_fps(subtitle.get_start(), src_fps, dest_fps);
		SubtitleTime end = change_fps(subtitle.get_end(), src_fps, dest_fps);

		subtitle.set_start_and_end(start, end);
	}
	
	doc.emit_signal("subtitle-time-changed");
	doc.flash_message(_("The new framerate was applied. (%s to %s)"), to_string(src_fps).c_str(), to_string(dest_fps).c_str());
	return true;
}

/*
 *
 */
SubtitleTime ChangeFramerate::change_fps(const SubtitleTime &time, double src, double dest)
{
	double frame = time.totalmsecs * src;
	
	double tot_sec = frame / dest;

	return SubtitleTime((long)tot_sec);
}
