/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2007, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include "SubtitleEditorProject.h"
#include <iostream>
#include <fstream>
#include <glibmm/ustring.h>
#include "utility.h"
#include "RegEx.h"
#include <libxml++/libxml++.h>

/*
 *
 */
Glib::ustring SubtitleEditorProject::get_name()
{
	return "Subtitle Editor Project";
}

/*
 *
 */
Glib::ustring SubtitleEditorProject::get_extension()
{
	return "sep";
}

/*
 *
 */
bool SubtitleEditorProject::check(const std::string &line)
{
	if(line.find("<SubtitleEditorProject") != std::string::npos)
		return true;
	return false;
}


/*
 *
 */
SubtitleEditorProject::SubtitleEditorProject(Document* doc)
:SubtitleFormat(doc, get_name())
{
	se_debug(SE_DEBUG_LOADER | SE_DEBUG_SAVER);
}

/*
 *
 */
SubtitleEditorProject::~SubtitleEditorProject()
{
	se_debug(SE_DEBUG_LOADER | SE_DEBUG_SAVER);
}


/*
 *
 */
bool SubtitleEditorProject::on_open(const Glib::ustring &filename)
{
	se_debug(SE_DEBUG_LOADER);

	try
	{
		xmlpp::DomParser parser;
		//parser.set_validate();
		parser.set_substitute_entities();
		parser.parse_file(filename);

		if(!parser)
			return false;

		const xmlpp::Node* root = parser.get_document()->get_root_node();

		// Styles
		

		// Subtitles
		const xmlpp::Element *xmlsubtitles = dynamic_cast<const xmlpp::Element*>(root->get_children("subtitles").front());

		const xmlpp::Node::NodeList list_subtitles = xmlsubtitles->get_children("subtitle");
		xmlpp::Node::NodeList::const_iterator it;

		for( it = list_subtitles.begin(); it != list_subtitles.end();	++it)
		{
			const xmlpp::Element *el = dynamic_cast<const xmlpp::Element*>(*it);

			SubtitleModifier sub(m_subtitleModel->append());

#define get_attribute(name) const xmlpp::Attribute *att = el->get_attribute(name)

			if(get_attribute("start"))		sub.set_start(SubtitleTime(att->get_value()));
			if(get_attribute("end"))			sub.set_end(SubtitleTime(att->get_value()));

			if(get_attribute("text"))					sub.set_text(att->get_value());
			if(get_attribute("translation"))	sub.set_translation(att->get_value());
			if(get_attribute("note"))					sub.set_note(att->get_value());

			if(get_attribute("layer"))		sub.set_layer(att->get_value());
			if(get_attribute("style"))		sub.set_style(att->get_value());
			if(get_attribute("name"))			sub.set_name(att->get_value());
			if(get_attribute("margin_l"))	sub.set_margin_l(att->get_value());
			if(get_attribute("margin_r"))	sub.set_margin_r(att->get_value());
			if(get_attribute("margin_v"))	sub.set_margin_v(att->get_value());
			if(get_attribute("effect"))		sub.set_effect(att->get_value());	
			
#undef get_attribute

		}

		return true;
	}
	catch(const std::exception &ex)
	{
		throw SubtitleException("SubtitleEditorProject", ex.what());
	}
	return false;
}

/*
 *
 */
bool SubtitleEditorProject::on_save(const Glib::ustring &filename)
{
	se_debug(SE_DEBUG_SAVER);

	try
	{
		xmlpp::Document xmldoc;

		xmlpp::Element *root = xmldoc.create_root_node("SubtitleEditorProject");
		root->set_attribute("version", "1.0");

		// Document

		// Style ------------------------
		xmlpp::Element *xmlstyles = root->add_child("styles");

		


		// Subtitle --------------------
		xmlpp::Element *xmlsubtitles = root->add_child("subtitles");

		Gtk::TreeNodeChildren rows = m_subtitleModel->children();

		for(Gtk::TreeIter it = rows.begin(); it; ++it)
		{
			xmlpp::Element *xml = xmlsubtitles->add_child("subtitle");

			SubtitleModifier sub(it);

			// time
			xml->set_attribute("start", sub.get_start().str());
			xml->set_attribute("end", sub.get_end().str());
			xml->set_attribute("duration", sub.get_duration().str());

			// text & translation
			xml->set_attribute("text", sub.get_text());
			xml->set_attribute("translation", sub.get_translation());
			xml->set_attribute("note", sub.get_note());

			// attribute (style, name, layer, margin...)
			xml->set_attribute("layer", sub.get_layer());
			xml->set_attribute("style", sub.get_style());
			xml->set_attribute("name", sub.get_name());
			xml->set_attribute("margin_l", sub.get_margin_l());
			xml->set_attribute("margin_r", sub.get_margin_r());
			xml->set_attribute("margin_v", sub.get_margin_v());
			xml->set_attribute("effect", sub.get_effect());

		}

		// TODO UTF-8
		xmldoc.write_to_file_formatted(filename, "UTF-8");

		return true;
	}
	catch(const std::exception &ex)
	{
		throw SubtitleException("SubtitleEditorProject", ex.what());
	}

	return false;
}




