/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include "SubtitleText.h"
#include <iostream>
#include <fstream>
#include "utility.h"
#include "RegEx.h"

/*
 *
 */
Glib::ustring SubtitleText::get_name()
{
	return "Text";
}

/*
 *
 */
Glib::ustring SubtitleText::get_extension()
{
	return "text";
}

/*
 *
 */
bool SubtitleText::check(const std::string &line)
{
	static RegEx ex("^.*?$");

	return ex.exec(line);
}

/*
 *
 */
SubtitleText::SubtitleText(Document *doc)
:SubtitleFormat(doc, get_name())
{
}

/*
 *
 */
SubtitleText::~SubtitleText()
{
}


/*
 *
 */
bool SubtitleText::on_open(const Glib::ustring &filename)
{
	se_debug(SE_DEBUG_LOADER);

	std::ifstream file(filename.c_str());
	if(!file)
	{
		return false;
	}

	std::string line;

	while(!file.eof() && std::getline(file, line))
	{
		if(line.size() > 0)
		{
			SubtitleModifier subtitle(m_subtitleModel->append());
			subtitle.set_text( check_end_char(charset_to_utf8(line)) );
		}
	}
	file.close();

	return true;
}

/*
 *
 */
bool SubtitleText::on_save(const Glib::ustring &filename)
{
	se_debug(SE_DEBUG_SAVER);

	std::ofstream file(filename.c_str());
	if(!file)
	{
		return false;
	}

	Gtk::TreeIter it = m_subtitleModel->getFirst();
	Glib::ustring line;

	for(; it; ++it)
	{
		SubtitleModifier subtitle(it);

		Glib::ustring text = subtitle.get_text();
		newline_to_characters(text, get_newline());

		file << utf8_to_charset(text);
		file << get_newline();
	}
	file.close();

	return true;
}
