#include "PreferencesUI.h"
#include "Config.h"
#include "utility.h"
#include "Encodings.h"

/*
 *	Interface
 */
class PreferenceInterfaceUI : public Gtk::VBox
{
public:
	PreferenceInterfaceUI(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::VBox(cobject)
	{
		refGlade->get_widget("check-use-dynamic-keyboard-shortcuts", m_checkUseDynamicKeyboardShortcuts);
		refGlade->get_widget("check-maximize-window", m_checkMaximizeWindow);
		refGlade->get_widget("check-ask-to-save-on-exit", m_checkAskToSaveOnExit);

		refGlade->get_widget("check-center-subtitle", m_checkCenterSubtitle);
		refGlade->get_widget("check-show-character-per-line", m_checkShowCharacterPerLine);

		refGlade->get_widget("check-create-backup-copy", m_checkCreateBackupCopy);
		refGlade->get_widget("check-autosave", m_checkAutosave);
		refGlade->get_widget("spin-autosave", m_spinAutosave);

		WidgetToConfig::read_config_and_connect(m_checkUseDynamicKeyboardShortcuts, "interface", "use-dynamic-keyboard-shortcuts", "true");
		WidgetToConfig::read_config_and_connect(m_checkMaximizeWindow, "interface", "maximize-window", "false");
		WidgetToConfig::read_config_and_connect(m_checkAskToSaveOnExit, "interface", "ask-to-save-on-exit", "false");

		WidgetToConfig::read_config_and_connect(m_checkCenterSubtitle, "subtitle-view", "property-alignment-center", "false");
		WidgetToConfig::read_config_and_connect(m_checkShowCharacterPerLine, "subtitle-view", "show-character-per-line", "true");
		
		WidgetToConfig::read_config_and_connect(m_checkCreateBackupCopy, "interface", "create-backup-copy", "false");
		WidgetToConfig::read_config_and_connect(m_checkAutosave, "interface", "used-autosave", "false");
		WidgetToConfig::read_config_and_connect(m_spinAutosave, "interface", "autosave-minutes", "10");

		m_checkCreateBackupCopy->set_sensitive(false);
	}


protected:
	Gtk::CheckButton* m_checkUseDynamicKeyboardShortcuts;
	Gtk::CheckButton* m_checkMaximizeWindow;
	Gtk::CheckButton* m_checkAskToSaveOnExit;

	Gtk::CheckButton* m_checkShowCharacterPerLine;
	Gtk::CheckButton* m_checkCenterSubtitle;

	Gtk::CheckButton* m_checkCreateBackupCopy;
	Gtk::CheckButton* m_checkAutosave;
	Gtk::SpinButton*	m_spinAutosave;
};

/*
 *	Encodings
 */
class PreferencesEncodingsUI : public Gtk::VBox
{
	class ColumnEncoding : public Gtk::TreeModel::ColumnRecord
	{
	public:
		ColumnEncoding()
		{
			add(use);
			add(name);
			add(charset);
		}
		Gtk::TreeModelColumn<Glib::ustring> use;
		Gtk::TreeModelColumn<Glib::ustring> name;
		Gtk::TreeModelColumn<Glib::ustring> charset;
	};

public:
	PreferencesEncodingsUI(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::VBox(cobject)
	{
		refGlade->get_widget("treeview-encodings", m_treeviewEncodings);
		refGlade->get_widget("button-add-charset", m_buttonAddCharset);
		refGlade->get_widget("button-remove-charset", m_buttonRemoveCharset);
		refGlade->get_widget("button-up-charset", m_buttonUpCharset);
		refGlade->get_widget("button-down-charset", m_buttonDownCharset);
		refGlade->get_widget("check-used-auto-detected", m_checkUsedAutoDetected);

		m_buttonAddCharset->signal_clicked().connect(
				sigc::mem_fun(*this, &PreferencesEncodingsUI::on_add_charset));
		m_buttonRemoveCharset->signal_clicked().connect(
				sigc::mem_fun(*this, &PreferencesEncodingsUI::on_remove_charset));
		m_buttonUpCharset->signal_clicked().connect(
				sigc::mem_fun(*this, &PreferencesEncodingsUI::on_up_charset));
		m_buttonDownCharset->signal_clicked().connect(
				sigc::mem_fun(*this, &PreferencesEncodingsUI::on_down_charset));

		WidgetToConfig::read_config_and_connect(m_checkUsedAutoDetected, "encodings", "used-auto-detected", "true");

		createTreeView(m_treeviewEncodings);

		load_config();
	}

	Glib::RefPtr<Gtk::ListStore> createTreeView(Gtk::TreeView *view)
	{
		ColumnEncoding m_column;
		Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(m_column);

		view->set_model(model);

		Gtk::TreeViewColumn* column = NULL;
		Gtk::CellRendererText* name = NULL;
		Gtk::CellRendererText* charset = NULL;

		// column name
		column = manage(new Gtk::TreeViewColumn(_("Name")));
		view->append_column(*column);
	
		name = manage(new Gtk::CellRendererText);
		column->pack_start(*name, false);
		column->add_attribute(name->property_text(), m_column.name);

		// column charset
		column = manage(new Gtk::TreeViewColumn(_("Charset")));
		view->append_column(*column);

		charset = manage(new Gtk::CellRendererText);
		column->pack_start(*charset, false);
		column->add_attribute(charset->property_text(), m_column.charset);

		return model;
	}

	void load_config()
	{
		Glib::RefPtr<Gtk::ListStore> model = Glib::RefPtr<Gtk::ListStore>::cast_dynamic(m_treeviewEncodings->get_model());
		
		ColumnEncoding m_column;
		
		// add encodings pref 
		std::list<Glib::ustring> list_encodings;
		if(Config::getInstance().get_value_string_list("encodings", "encodings", list_encodings))
		{
			std::list<Glib::ustring>::const_iterator it;
			for(it = list_encodings.begin(); it!=list_encodings.end(); ++it)
			{
				EncodingInfo *info= Encodings::get_from_charset(*it);
				if(info)
				{
					Gtk::TreeIter it = model->append();

					(*it)[m_column.name] = info->name;
					(*it)[m_column.charset] = info->charset;
				}
			}
		}
	}

	void save_config()
	{
		Glib::RefPtr<Gtk::ListStore> m_model = 
			Glib::RefPtr<Gtk::ListStore>::cast_dynamic(m_treeviewEncodings->get_model());

		Config &cfg = Config::getInstance();

		std::list<Glib::ustring> list;
		
		ColumnEncoding m_column;
	
		Gtk::TreeNodeChildren rows = m_model->children();

		for(Gtk::TreeIter iter = rows.begin(); iter; ++iter)
			list.push_back((*iter)[m_column.charset]);

		cfg.set_value_string_list("encodings", "encodings", list);
	}

	void on_add_charset()
	{
		Gtk::Dialog *dialog = NULL;
		Gtk::TreeView* treeview = NULL;

		Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(get_share_dir("dialog-encodings-chooser.glade"));
	
		refXml->get_widget("dialog-encodings-chooser", dialog);
		refXml->get_widget("treeviewEncodings", treeview);

		Glib::RefPtr<Gtk::ListStore> model = createTreeView(treeview);
		ColumnEncoding m_column;
		
		// add encodings
		for(unsigned int i=0; encodings_info[i].charset != NULL; ++i)
		{
			Gtk::TreeIter it = model->append();
		
			EncodingInfo *info= Encodings::get_from_charset(encodings_info[i].charset);
			if(info)
			{
				(*it)[m_column.name] = info->name;
				(*it)[m_column.charset] = info->charset;
			}
		}

		if( dialog->run() == Gtk::RESPONSE_OK)
		{
			Gtk::TreeIter iter = treeview->get_selection()->get_selected();
		
			if(iter)
			{
				Glib::ustring name = (*iter)[m_column.name];
				Glib::ustring charset = (*iter)[m_column.charset];
				// add in the list
				Glib::RefPtr<Gtk::ListStore> modelEncodings = 
					Glib::RefPtr<Gtk::ListStore>::cast_dynamic(m_treeviewEncodings->get_model());
				
				Gtk::TreeIter it = modelEncodings->append();
				(*it)[m_column.name] = name;
				(*it)[m_column.charset] = charset;
			}
		}

		delete dialog;

		save_config();
	}

	void on_remove_charset()
	{
		Glib::RefPtr<Gtk::ListStore> modelEncodings = 
			Glib::RefPtr<Gtk::ListStore>::cast_dynamic(m_treeviewEncodings->get_model());

		Gtk::TreeIter iter = m_treeviewEncodings->get_selection()->get_selected();

		if(iter)
		{
			modelEncodings->erase(iter);
			save_config();
		}
	}

	void on_up_charset()
	{
		Glib::RefPtr<Gtk::ListStore> m_model = 
			Glib::RefPtr<Gtk::ListStore>::cast_dynamic(m_treeviewEncodings->get_model());

		Gtk::TreeIter it = m_treeviewEncodings->get_selection()->get_selected();
		if(it)
		{
			Gtk::TreePath path = m_model->get_path(it);

			if(path.prev())
			{
				Gtk::TreeIter prev = m_model->get_iter(path);
				if(prev)
				{
					m_model->move(it, prev);
					save_config();
				}
			}
		}
	}

	void on_down_charset()
	{
		Glib::RefPtr<Gtk::ListStore> m_model = 
			Glib::RefPtr<Gtk::ListStore>::cast_dynamic(m_treeviewEncodings->get_model());

		Gtk::TreeIter it = m_treeviewEncodings->get_selection()->get_selected();
		if(it)
		{
			Gtk::TreeIter next = it;
			++next;
			if(next)
			{
				m_model->move(next, it);
				save_config();
			}
		}
	}

protected:
	Gtk::TreeView* m_treeviewEncodings;
	Gtk::Button* m_buttonAddCharset;
	Gtk::Button* m_buttonRemoveCharset;
	Gtk::Button* m_buttonUpCharset;
	Gtk::Button* m_buttonDownCharset;
	Gtk::CheckButton* m_checkUsedAutoDetected;
};

struct t_output
{
	const gchar *label;
	const gchar *name;
};

t_output m_audio_output[]={
	{_("Autodetect"), "autoaudiosink"},
	{_("ALSA - Advanced Linux Sound Architecture"), "alsasink"},
	{_("ESD - Enlightenment Sound Daemon"), "esdsink"},
	{_("OSS - Open Sound System"), "ossink"},
	{_("SDL - Simple DirectMedia Layer"), "sdlaudiosink"},
	{_("GConf"), "gconfaudiosink"},
	{NULL, NULL}
};

t_output m_video_output[]={
	{_("Autodetect"), "autovideosink"},
	{_("X Window System (X11/XShm/Xv)"), "xvimagesink"},
	{_("X Window System (No Xv)"), "ximagesink"},
	{_("SDL - Simple DirectMedia Layer"), "sdlvideosink"},
	{_("GConf"), "gconfvideosink"},
	{_("OpenGL "), "glimagesink"},
	{NULL, NULL}
};

/*
 *	Video Player (gstreamer)
 *	Subtitle font, use shaded background, color blance
 */
class PreferenceVideoPlayerUI : public Gtk::VBox
{
public:
	PreferenceVideoPlayerUI(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::VBox(cobject)
	{
		refGlade->get_widget("fontbutton-subtitle", m_fontbuttonSubtitle);
		refGlade->get_widget("check-use-shaded-background", m_checkUseShadedBackground);
		refGlade->get_widget("check-display-translated-subtitle", m_checkDisplayTranslatedSubtitle);

		refGlade->get_widget("check-force-aspect-ratio", m_checkForceAspectRatio);
		
		refGlade->get_widget("hscale-brightness", m_hscaleBrightness);
		refGlade->get_widget("hscale-contrast", m_hscaleContrast);
		refGlade->get_widget("hscale-saturation", m_hscaleSaturation);
		refGlade->get_widget("hscale-hue", m_hscaleHue);
		refGlade->get_widget("button-reset-color-balance", m_buttonResetColorBalance);

		refGlade->get_widget_derived("combo-audio-output", m_comboAudioOutput);
		refGlade->get_widget_derived("combo-video-output", m_comboVideoOutput);
		
		refGlade->get_widget("spin-very-short-jump-lenght", m_spinVeryShortJumpLenght);
		refGlade->get_widget("spin-short-jump-lenght", m_spinShortJumpLenght);
		refGlade->get_widget("spin-medium-jump-lenght", m_spinMediumJumpLenght);
		refGlade->get_widget("spin-long-jump-lenght", m_spinLongJumpLenght);

		for(unsigned int i=0; m_audio_output[i].label != NULL; ++i)
		{
			m_comboAudioOutput->append_text(m_audio_output[i].label);
		}

		for(unsigned int i=0; m_video_output[i].label != NULL; ++i)
		{
			m_comboVideoOutput->append_text(m_video_output[i].label);
		}

		WidgetToConfig::read_config_and_connect(m_checkUseShadedBackground, "video-player", "shaded-background", "true");
		WidgetToConfig::read_config_and_connect(m_fontbuttonSubtitle, "video-player", "font-desc", "Sans 26");
		WidgetToConfig::read_config_and_connect(m_checkDisplayTranslatedSubtitle, "video-player", "display-translated-subtitle", "true");

		WidgetToConfig::read_config_and_connect(m_checkForceAspectRatio, "video-player", "force-aspect-ratio", "true");

		WidgetToConfig::read_config_and_connect(m_hscaleBrightness, "video-player", "brightness", "0");
		WidgetToConfig::read_config_and_connect(m_hscaleContrast, "video-player", "contrast", "0");
		WidgetToConfig::read_config_and_connect(m_hscaleSaturation, "video-player", "saturation", "0");
		WidgetToConfig::read_config_and_connect(m_hscaleHue, "video-player", "hue", "0");

		WidgetToConfig::read_config_and_connect(m_spinVeryShortJumpLenght, "video-player", "jump-very-short", "3");
		WidgetToConfig::read_config_and_connect(m_spinShortJumpLenght, "video-player", "jump-short", "10");
		WidgetToConfig::read_config_and_connect(m_spinMediumJumpLenght, "video-player", "jump-medium", "60");
		WidgetToConfig::read_config_and_connect(m_spinLongJumpLenght, "video-player", "jump-long", "300");

		m_buttonResetColorBalance->signal_clicked().connect(
				sigc::mem_fun(*this, &PreferenceVideoPlayerUI::on_reset_color_balance));

		Glib::ustring audiosink, videosink;
		Config::getInstance().get_value_string("video-player", "audio-sink", audiosink, "autoaudiosink");
		Config::getInstance().get_value_string("video-player", "video-sink", videosink, "xvimagesink");

		m_comboAudioOutput->set_active_text(get_output_label_by_name(m_audio_output, audiosink));
		m_comboVideoOutput->set_active_text(get_output_label_by_name(m_video_output, videosink));

		m_comboAudioOutput->signal_changed().connect(
				sigc::mem_fun(*this, &PreferenceVideoPlayerUI::on_audio_output_changed));
		m_comboVideoOutput->signal_changed().connect(
				sigc::mem_fun(*this, &PreferenceVideoPlayerUI::on_video_output_changed));
	}

protected:

	Glib::ustring get_output_label_by_name(t_output output[], const Glib::ustring &name)
	{
		for(unsigned int i=0; output[i].name != NULL; ++i)
		{
			if(name == output[i].name)
				return output[i].label;
		}
		return Glib::ustring();
	}

	Glib::ustring get_output_name_by_label(t_output output[], const Glib::ustring &label)
	{
		for(unsigned int i=0; output[i].name != NULL; ++i)
		{
			if(label == output[i].label)
				return output[i].name;
		}
		return Glib::ustring();
	}

	void on_reset_color_balance()
	{
		m_hscaleBrightness->set_value(0);
		m_hscaleContrast->set_value(0);
		m_hscaleSaturation->set_value(0);
		m_hscaleHue->set_value(0);
	}

	void on_audio_output_changed()
	{
		Glib::ustring name = get_output_name_by_label(m_audio_output, m_comboAudioOutput->get_active_text());
		Config::getInstance().set_value_string("video-player", "audio-sink", name);
	}

	void on_video_output_changed()
	{
		Glib::ustring name = get_output_name_by_label(m_video_output, m_comboVideoOutput->get_active_text());
		Config::getInstance().set_value_string("video-player", "video-sink", name);
	}
protected:
	Gtk::FontButton* m_fontbuttonSubtitle;
	Gtk::CheckButton* m_checkUseShadedBackground;
	Gtk::CheckButton* m_checkDisplayTranslatedSubtitle;

	Gtk::CheckButton* m_checkForceAspectRatio;
	Gtk::HScale*	m_hscaleBrightness;
	Gtk::HScale*	m_hscaleContrast;
	Gtk::HScale*	m_hscaleSaturation;
	Gtk::HScale*	m_hscaleHue;
	Gtk::Button*	m_buttonResetColorBalance;

	ComboBoxText*	m_comboAudioOutput;
	ComboBoxText*	m_comboVideoOutput;

	Gtk::SpinButton* m_spinVeryShortJumpLenght;
	Gtk::SpinButton* m_spinShortJumpLenght;
	Gtk::SpinButton* m_spinMediumJumpLenght;
	Gtk::SpinButton* m_spinLongJumpLenght;
};


/*
 *	Waveform Preference
 *	Color, Waveform Generator
 */
class PreferenceWaveformUI : public Gtk::VBox
{
public:
	PreferenceWaveformUI(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::VBox(cobject)
	{
		se_debug(SE_DEBUG_APP);

		refGlade->get_widget("colorbutton-background", m_colorbuttonBackground);
		refGlade->get_widget("colorbutton-play-line", m_colorbuttonPlayLine);
		refGlade->get_widget("colorbutton-wave", m_colorbuttonWave);
		refGlade->get_widget("colorbutton-wave-fill", m_colorbuttonWaveFill);
		refGlade->get_widget("colorbutton-marker", m_colorbuttonMarker);
		refGlade->get_widget("colorbutton-marker-hightlight", m_colorbuttonMarkerHightlight);
		refGlade->get_widget("check-display-background", m_checkDisplayBackground);
		refGlade->get_widget("check-display-waveform-fill", m_checkDisplayWaveformFill);
		refGlade->get_widget("button-reset-to-defaults-waveform-color", m_buttonReset);

		WidgetToConfig::read_config_and_connect(m_colorbuttonBackground, "waveform", "color-background", 
				build_message("%f;%f;%f;%f", 0.8, 0.8, 0.8, 1.0));
		WidgetToConfig::read_config_and_connect(m_colorbuttonPlayLine, "waveform", "color-play-line", 
				build_message("%f;%f;%f;%f", 0.0, 0.0, 0.0, 1.0));
		WidgetToConfig::read_config_and_connect(m_colorbuttonWave, "waveform", "color-wave", 
				build_message("%f;%f;%f;%f", 0.0, 0.7, 0.0, 1.0));
		WidgetToConfig::read_config_and_connect(m_colorbuttonWaveFill, "waveform", "color-wave-fill", 
				build_message("%f;%f;%f;%f", 0.0, 0.5, 0.0, 1.0));
		WidgetToConfig::read_config_and_connect(m_colorbuttonMarker, "waveform", "color-marker", 
				build_message("%f;%f;%f;%f", 0.4, 0.4, 0.4, 0.5));
		WidgetToConfig::read_config_and_connect(m_colorbuttonMarkerHightlight, "waveform", "color-marker-hightlight", 
				build_message("%f;%f;%f;%f", 0.3, 0.5, 0.8, 0.5));
		WidgetToConfig::read_config_and_connect(m_checkDisplayBackground, "waveform", "display-background", "false");
		WidgetToConfig::read_config_and_connect(m_checkDisplayWaveformFill, "waveform", "display-waveform-fill", "true");

		m_buttonReset->signal_clicked().connect(
				sigc::mem_fun(*this, &PreferenceWaveformUI::on_reset));
	}

protected:

	void on_reset()
	{
		se_debug(SE_DEBUG_APP);

#define SET_COLOR(w, key, r,g,b,a) \
	{ \
		Gdk::Color color; color.set_rgb_p(r,g,b); w->set_color(color); w->set_alpha((guint64)(a * 65535)); \
		Config::getInstance().set_value_string("waveform", key, build_message("%f;%f;%f;%f", r,g,b,a)); \
	}
		SET_COLOR(m_colorbuttonBackground, "color-background", 0.8, 0.8, 0.8, 1.0);
		SET_COLOR(m_colorbuttonPlayLine, "color-play-line", 0.0, 0.0, 0.0, 1.0);
		SET_COLOR(m_colorbuttonWave, "color-wave", 0.0, 0.7, 0.0, 1.0);
		SET_COLOR(m_colorbuttonWaveFill, "color-wave-fill", 0.0, 0.5, 0.0, 1.0);
		SET_COLOR(m_colorbuttonMarker, "color-marker", 0.4, 0.4, 0.4, 0.5);
		SET_COLOR(m_colorbuttonMarkerHightlight, "color-marker-hightlight", 0.3, 0.5, 0.8, 0.5);
#undef SET_COLOR

	}
protected:
	Gtk::ColorButton*	m_colorbuttonBackground;
	Gtk::ColorButton*	m_colorbuttonPlayLine;
	Gtk::ColorButton*	m_colorbuttonWave;
	Gtk::ColorButton*	m_colorbuttonWaveFill;
	Gtk::ColorButton*	m_colorbuttonMarker;
	Gtk::ColorButton*	m_colorbuttonMarkerHightlight;

	Gtk::CheckButton* m_checkDisplayBackground;
	Gtk::CheckButton* m_checkDisplayWaveformFill;

	Gtk::Button*			m_buttonReset;
};


/*
 *	Timing/Synchro
 */
class PreferenceTimingUI : public Gtk::VBox
{
public:
	PreferenceTimingUI(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::VBox(cobject)
	{
		refGlade->get_widget("spin-min-characters-per-second", m_spinMinCharactersPerSecond);
		refGlade->get_widget("spin-max-characters-per-second", m_spinMaxCharactersPerSecond);
		refGlade->get_widget("spin-min-gap-between-subtitle", m_spinMinGapBetweenSubtitle);
		refGlade->get_widget("spin-min-display", m_spinMinDisplay);
		refGlade->get_widget("spin-max-characters-per-line", m_spinMaxCharactersPerLine);
		refGlade->get_widget("spin-max-line", m_spinMaxLine);

		WidgetToConfig::read_config_and_connect(m_spinMinCharactersPerSecond, "timing", "min-characters-per-second", "5");
		WidgetToConfig::read_config_and_connect(m_spinMaxCharactersPerSecond, "timing", "max-characters-per-second", "25");
		WidgetToConfig::read_config_and_connect(m_spinMinGapBetweenSubtitle, "timing", "min-gap-between-subtitle", "100");
		WidgetToConfig::read_config_and_connect(m_spinMinDisplay, "timing", "min-display", "1000");
		WidgetToConfig::read_config_and_connect(m_spinMaxCharactersPerLine, "timing", "max-characters-per-line", "40");
		WidgetToConfig::read_config_and_connect(m_spinMaxLine, "timing", "max-line", "2");
	}

protected:
	Gtk::SpinButton* m_spinMinCharactersPerSecond;
	Gtk::SpinButton* m_spinMaxCharactersPerSecond;
	Gtk::SpinButton* m_spinMinGapBetweenSubtitle;
	Gtk::SpinButton* m_spinMinDisplay;
	Gtk::SpinButton* m_spinMaxCharactersPerLine;
	Gtk::SpinButton* m_spinMaxLine;
};

/*
 *
 */
class PreferencePreviewUI : public Gtk::VBox
{
public:
	PreferencePreviewUI(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::VBox(cobject)
	{
		refGlade->get_widget("entry-video-player-command", m_entryVideoPlayerCommand);
		
		Glib::ustring default_player = "mplayer \"#video_file\" -sub \"#subtitle_file\" -ss #seconds -osdlevel 2";

		WidgetToConfig::read_config_and_connect(m_entryVideoPlayerCommand, "external-video-player", "command", default_player);
	}
protected:
	Gtk::Entry* m_entryVideoPlayerCommand;
};



/*
 *
 */
PreferencesUI::PreferencesUI(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
:Gtk::Dialog(cobject)
{
	PreferenceInterfaceUI *interfaceUI = NULL;
	PreferencesEncodingsUI *encodingsUI = NULL;
	PreferenceWaveformUI *waveformUI = NULL;
	PreferenceVideoPlayerUI *videoplayerUI = NULL;
	PreferenceTimingUI *timingUI = NULL;
	PreferencePreviewUI *previewUI = NULL;

	refGlade->get_widget_derived("vbox-interface", interfaceUI);
	refGlade->get_widget_derived("vbox-encodings", encodingsUI);
	refGlade->get_widget_derived("vbox-waveform", waveformUI);
	refGlade->get_widget_derived("vbox-video-player", videoplayerUI);
	refGlade->get_widget_derived("vbox-timing", timingUI);
	refGlade->get_widget_derived("vbox-preview", previewUI);
}
